# -*- encoding: utf-8 -*-
# stub: scheduled_resource 0.0.3 ruby lib

Gem::Specification.new do |s|
  s.name = "scheduled_resource".freeze
  s.version = "0.0.3"

  s.required_rubygems_version = Gem::Requirement.new(">= 0".freeze) if s.respond_to? :required_rubygems_version=
  s.require_paths = ["lib".freeze]
  s.authors = ["Mike Cannon".freeze]
  s.date = "2015-03-16"
  s.description = "== README.md:\n#ScheduledResource\n\nThis gem is for displaying how things are used\nover time -- a schedule for a set of \"resources\".  You\ncan configure the elements of the schedule and there\nare utilities and protocols to connect them:\n\n - Configuration (specification and management),\n - Query interfaces (a REST-like API and internal protocols to query the models), and\n - A basic Rails controller implementation.\n\nWe have a way to configure the schedule, internal\nmethods to generate the data, and a way to retrieve\ndata from the client.  However this gem is largely\nview-framework agnostic.  We could use a variety of\nclient-side packages or even more traditional Rails\nview templates to generate HTML.\n\nIn any case, to get a good feel in a display like\nthis we need some client-side code.  The gem includes\nclient-side modules to:\n\n - Manage <b>time and display geometries</b> with \"infinite\" scroll along the time axis.\n - <b>Format display cells</b> in ways specific to the resource models.\n - <b>Update text justification</b> as the display is scrolled horizontally.\n\n\n## Configuration\n\nA **scheduled resource** is something that can be\nused for one thing at a time.  So if \"Rocky & Bullwinkle\"\nis on channel 3 from 10am to 11am on Saturday, then\n'channel 3' is the <u>resource</u> and that showing of\nthe episode is a <u>resource-use</u> block.  Resources \nand use-blocks are typically Rails models.  Each resource\nand its use-blocks get one row in the display.  That\nrow has a label to the left with some timespan visible\non the rest of the row.\n\nSomething else you would expect see in a schedule\nwould be headers and labels -- perhaps one row with\nthe date and another row with the hour.  Headers and\nlabels also fit the model of resources and use-blocks.\nBasic timezone-aware classes (ZTime*) for those are\nincluded in this gem.\n\n\n### Config File\n\nThe schedule configuration comes from\n<tt>config/resource_schedule.yml</tt> which has\nthree top-level sections:\n\n- ResourceKinds:  A hash where the key is a Resource and the value is a UseBlock. (Both are class names),\n- Resources:  A list where each item is a Resource Class followed by one or more resource ids, and\n- visibleTime:  The visible timespan of the schedule in seconds.\n\nThe example file <tt>config/resource_schedule.yml</tt>\n(installed when you run <tt>schedulize</tt>) should be\nenough to display a two-row schedule with just the date\nabove and the hour below.  Of course you can monkey-patch\nor subclass these classes for your own needs.\n\n\n### The schedule API\n\nThe 'schedule' endpoint uses parameters <tt>t1</tt> and\n<tt>t2</tt> to specify a time interval for the request.\nA third parameter <tt>inc</tt> allows an initial time\nwindow to be expanded without repeating blocks that\nspan those boundaries.  The time parameters\n_plus the configured resources_ define the data to be returned.\n\n\n### More About Configuration Management\n\nThe <b>ScheduledResource</b> class manages resource and\nuse-block class names, id's and labels for a schedule\naccording to the configuration file.\nA ScheduledResource instance ties together:\n\n 1. A resource class (eg TvStation),\n 2. An id (a channel number in this example), and\n 3. Strings and other assets that will go into the DOM.\n\nThe id is used to\n  - select a resource _instance_ and\n  - select instances of the _resource use block_ class (eg Program instances).\n\nThe id _could_ be a database id but more\noften is something a little more suited to human use\nin the configuration.  In any case it is used by model\nclass method\n<tt>(resource_use_block_class).get_all_blocks()</tt>\nto select the right use-blocks for the resource.\nA resource class name and id are are joined with\na '_' to form a tag that also serves as an id for the DOM.\n\nOnce the configuration yaml is loaded that data is\nmaintained in the session structure.  Of course having\na single configuration file limits the application's\nusefulness.  A more general approach would be to\nhave a user model with login and configuration would\nbe associated with the user.\n\n\n## Installation\n\nAdd this line to your application's Gemfile:\n\n```ruby\ngem 'scheduled_resource'\n```\n\nAnd then execute:\n\n    $ bundle\n\nOr install it yourself as:\n\n    $ gem install scheduled_resource\n\nThen from your application's root execute:\n\n    $ schedulize .\n\nThis will install a few image placeholders, \nclient-side modules and a stylesheet under \n<tt>vendor/assets</tt>, an example configuration\nin <tt>config/resource_schedule.yml</tt> and\nan example controller in\n<tt>app/controllers/schedule_controller.rb</tt>.\n\nAlso, if you use\n\n    $ bundle show scheduled_resource\n\nto locate the installed source you can browse\nexample classes <tt>lib/z_time_*.rb</tt> and\nthe controller helper methods in\n<tt>lib/scheduled_resource/helper.rb</tt>\n\n\n## Testing\n\nThis gem also provides for a basic test application\nusing angularjs to display a minimal but functional\nschedule showing just the day and hour headers in\ntwo different timezones (US Pacific and Eastern).\nProceed as follows, starting with a fresh Rails app:\n\n    $ rails new test_sr\n\nAs above, add the gem to the Gemfile, then \n\n    $ cd test_sr\n    $ bundle\n    $ schedulize .\n\nAdd lines such as these to <tt>config/routes.rb</tt>\n\n    get \"/schedule/index\" => \"schedule#index\"\n    get \"/schedule\"       => \"schedule#schedule\"\n\nCopy / merge these files from the gem source into\nthe test app:\n\n    $SR_SRC/app/views/layouts/application.html.erb\n    $SR_SRC/app/views/schedule/index.html.erb\n    $SR_SRC/app/assets/javascripts/{angular.js,script.js,controllers.js}\n\nand add <tt>//= require angular</tt> to application.js\njust below the entries for <tt>jquery</tt>.\n\nAfter you run the server and browse to\n\n    http://0.0.0.0:3000/schedule/index\n\nyou should see the four time-header rows specified\nby the sample config file.\n\n\n## More Examples\n\nA better place to see the use of this gem is at\n[tv4](https://github.com/emeyekayee/tv4).  Specifically,\nmodels <tt>app/models/event.rb</tt> and\n<tt>app/models/station.rb</tt> give better examples of\nimplementing the ScheduledResource protocol and adapting\nto a db schema organized along somewhat different lines.\n\n\n\n\n## Contributing\n\n1. Fork it ( https://github.com/emeyekayee/scheduled_resource/fork )\n2. Create your feature branch (`git checkout -b my-new-feature`)\n3. Commit your changes (`git commit -am 'Add some feature'`)\n4. Push to the branch (`git push origin my-new-feature`)\n5. Create a new Pull Request\n".freeze
  s.email = ["michael.j.cannon@gmail.com".freeze]
  s.executables = ["schedulize".freeze]
  s.files = ["bin/schedulize".freeze]
  s.homepage = "http://github.com/emeyekayee/scheduled_resource".freeze
  s.licenses = ["MIT".freeze]
  s.rubygems_version = "3.3.7".freeze
  s.summary = "Display how something is used over time.".freeze

  s.installed_by_version = "3.3.7" if s.respond_to? :installed_by_version

  if s.respond_to? :specification_version then
    s.specification_version = 4
  end

  if s.respond_to? :add_runtime_dependency then
    s.add_runtime_dependency(%q<rails>.freeze, ["~> 4.2"])
    s.add_runtime_dependency(%q<coffee-rails>.freeze, ["~> 4.1"])
    s.add_development_dependency(%q<bundler>.freeze, ["~> 1.7"])
    s.add_development_dependency(%q<rake>.freeze, ["~> 10.0"])
  else
    s.add_dependency(%q<rails>.freeze, ["~> 4.2"])
    s.add_dependency(%q<coffee-rails>.freeze, ["~> 4.1"])
    s.add_dependency(%q<bundler>.freeze, ["~> 1.7"])
    s.add_dependency(%q<rake>.freeze, ["~> 10.0"])
  end
end
